//---------------------------------------------------------------------------
#ifndef GLRenderH
#define GLRenderH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <gl\gl.h>
#include <gl\glu.h>



#include "BMPTexture.hpp"
#include "OpenGLFont3D.hpp"
#include "OpenGLFont2D.hpp"
#include "PowerColor.h"

typedef void __fastcall (__closure *TParamsEvent)(System::TObject* Sender , TCreateParams &Params );

//---------------------------------------------------------------------------
class PACKAGE TCustomGLRender : public TCustomPanel
{
private:
 //Enumerations
 enum TPFDPixelTypes     {f_PFD_TYPE_RGBA,f_PFD_TYPE_COLORINDEX};
 enum TPFDFlags          {f_PFD_DRAW_TO_WINDOW, f_PFD_DRAW_TO_BITMAP,
                         f_PFD_SUPPORT_GDI, f_PFD_SUPPORT_OPENGL,
                         f_PFD_GENERIC_ACCELERATED, f_PFD_GENERIC_FORMAT,
                         f_PFD_NEED_PALETTE, f_PFD_NEED_SYSTEM_PALETTE,
                         f_PFD_DOUBLEBUFFER,f_PFD_STEREO,
                         f_PFD_SWAP_LAYER_BUFFERS};
 enum TOpenGLSwapBuffers {Auto,Manual};
 enum TOpenGLPrintScale  {pglNone,pglPrintToFit,pglProportional};
 enum TFont3DFormat      {f_WGL_FONT_LINES,f_WGL_FONT_POLYGONS};

 // types
 typedef Set<TPFDFlags, f_PFD_DRAW_TO_WINDOW, f_PFD_SWAP_LAYER_BUFFERS> TPFDFlagsSet;
 // variables
   HDC                   DisplayDeviceContext;
   HGLRC                 GLRenderingContext;
   HPALETTE              Palette;

   TPFDFlagsSet          FdwFlags;
   bool                  DoubleBuffer;
   bool                  AutoSwapbuffers;
   TPFDPixelTypes        FiPixelType;
   BYTE                  SavedPixelType;
   BYTE                  FcColorBits;
   BYTE                  FcDepthBits;
   TOpenGLSwapBuffers    FSwapBuffers;

   TOpenGLPrintScale     FOpenGLPrintScale;
   int                   FOpenGLPixelsPerInch;

   bool                  Font3DEnabled;
   TFont               * Font3DType;
   int                   Font3DFirstGylph;
   int                   Font3DNumGylph;
   float                 Font3DMaxDeviation;
   float                 Font3DExtrusion;
   TFont3DFormat         Font3DFormat;
   int                   SavedFont3DFormat;
   OpenGLFont3D        * Font3DDefault;

   bool                  Font2DEnabled;
   TFont               * Font2DType;
   int                   Font2DFirstGylph;
   int                   Font2DNumGylph;
   OpenGLFont2D        * Font2DDefault;

   AnsiString          FGLExtensions          ;
   AnsiString          FWGLExtensions         ;
   AnsiString          FSupportExtensions     ;
   AnsiString          FUnsupportExtensions   ;
   AnsiString          FUserRequestExtensions ;

   AnsiString FGLRenderer;
   AnsiString FGLVersion;
   AnsiString FGLVendor;

   // Methods
              bool   __fastcall CreateGLContext(void);
              void   __fastcall DestroyGLContext(void);
              bool   __fastcall ConstructPalette(PIXELFORMATDESCRIPTOR &pfd);
              void   __fastcall Create3DFont(void);
              void   __fastcall Create2DFont(void);

   // Property Medthods
              void   __fastcall SetPFDPixelType(const TPFDPixelTypes Value);
              void   __fastcall SetPFDFlags(const TPFDFlagsSet Value);
              void   __fastcall SetPFDNumColorBits(const BYTE Value);
              void   __fastcall SetPFDNumDepthBits(const BYTE Value);

              void   __fastcall SetOpenGLPrintScale(const TOpenGLPrintScale Value);
              void   __fastcall SetOpenGLPixelsPerInch(const int Value);

              void   __fastcall SetFont3DType(TFont* Value);
              void   __fastcall SetFont3DFormat(const TFont3DFormat Value);
              void   __fastcall SetFont(TFont* Value);

              void   __fastcall ReadGLExtensions();
              void   __fastcall ReadWGLExtensions();
              void   __fastcall ReadSupportExtensions() ;

              AnsiString __fastcall GetGLExtensions();
              AnsiString __fastcall GetWGLExtensions();
              AnsiString __fastcall GetUnsupportExtensions();

              AnsiString __fastcall GetGLRenderer();
              AnsiString __fastcall GetGLVersion();
              AnsiString __fastcall GetGLVendor();

              bool   __fastcall GetIsCurrent() ;
              bool   __fastcall GetIsCreated() ;

protected:
   // Variables
   TNotifyEvent FOnPaint            ;
   TNotifyEvent FOnResize           ;
   TNotifyEvent FOnRenderInit       ;
   TNotifyEvent FOnExtensionsInit   ;
   TNotifyEvent FOnResetDisplayMode ;
   TNotifyEvent FOnSetDisplayMode   ;
   TParamsEvent FOnSetParams        ;

   // Inherited or Overidden Methods
   virtual    void   __fastcall CreateParams(TCreateParams &Params);
   virtual    void   __fastcall Paint();
   DYNAMIC    void   __fastcall Resize();
              void   __fastcall CreateRender(TWMCreate &Message);
              void   __fastcall WMPaletteChanged(TWMPaletteChanged &Message);
              void   __fastcall WMQueryNewPalette(TWMQueryNewPalette &Message);
              void   __fastcall WMEraseBkgnd(TWMEraseBkgnd &Message);
              void   __fastcall DestroyRender(TWMDestroy &Message);
   // Message Maps
   BEGIN_MESSAGE_MAP
      MESSAGE_HANDLER(WM_CREATE,TWMCreate,CreateRender)
      MESSAGE_HANDLER(WM_DESTROY,TWMDestroy,DestroyRender)
      MESSAGE_HANDLER(WM_PALETTECHANGED,TWMPaletteChanged,WMPaletteChanged)
      MESSAGE_HANDLER(WM_QUERYNEWPALETTE,TWMQueryNewPalette,WMQueryNewPalette)
      MESSAGE_HANDLER(WM_ERASEBKGND,TWMEraseBkgnd,WMEraseBkgnd)
   END_MESSAGE_MAP(TCustomPanel)

   __property AnsiString GLExtensions  = { read=GetGLExtensions };
   __property AnsiString WGLExtensions  = { read=GetWGLExtensions };
   __property AnsiString UnsupportExtensions = { read=GetUnsupportExtensions }  ;
   __property AnsiString SupportExtensions  = { read=FSupportExtensions };
   __property AnsiString UserRequestExtensions  = { read=FUserRequestExtensions , write=FUserRequestExtensions };

   __property AnsiString GLRenderer  = { read=GetGLRenderer };
   __property AnsiString GLVersion  = { read=GetGLVersion };
   __property AnsiString GLVendor  = { read=GetGLVendor };


  // Added Properites
  __property TPFDPixelTypes     PixelType = {read= FiPixelType, write= SetPFDPixelType, default=f_PFD_TYPE_RGBA};
  __property TPFDFlagsSet       PFDFlags = {read=FdwFlags, write=SetPFDFlags,nodefault};
  __property BYTE               cColorBits = {read= FcColorBits, write=SetPFDNumColorBits, default=32};
  __property BYTE               cDepthBits = {read= FcDepthBits, write=SetPFDNumDepthBits, default=32};
  __property TOpenGLSwapBuffers GLSwapBuffers ={read=FSwapBuffers, write=FSwapBuffers, default=Auto};
  __property int                PixelsPerInch = {read=FOpenGLPixelsPerInch, write= SetOpenGLPixelsPerInch,default=96};
  __property TOpenGLPrintScale  PrintScale = {read= FOpenGLPrintScale, write=SetOpenGLPrintScale, default=pglProportional};
  __property bool               Font3D_Enabled = {read=Font3DEnabled, write=Font3DEnabled, default=false};
  __property TFont            * Font3D_Type = {read=Font3DType, write=SetFont3DType};
  __property int                Font3D_FirstGylph = {read=Font3DFirstGylph, write=Font3DFirstGylph, default=32};
  __property int                Font3D_NumGylph = {read=Font3DNumGylph, write=Font3DNumGylph, default=96};
  __property float              Font3D_MaxDeviation = {read=Font3DMaxDeviation, write=Font3DMaxDeviation};
  __property float              Font3D_Extrustion = {read=Font3DExtrusion, write=Font3DExtrusion};
  __property TFont3DFormat      Font3D_Format = {read=Font3DFormat, write=SetFont3DFormat, default=f_WGL_FONT_POLYGONS};
  __property bool               Font2D_Enabled = {read=Font2DEnabled, write=Font2DEnabled, default=false};
  __property TFont              * Font2D_Type = {read=Font2DType, write=SetFont};
  __property int                Font2D_FirstGylph = {read=Font2DFirstGylph, write=Font2DFirstGylph, default=32};
  __property int                Font2D_NumGylph = {read=Font2DNumGylph, write=Font2DNumGylph, default=96};
  __property bool               IsCurrent = {read = GetIsCurrent } ;
  __property bool               IsCreated = {read = GetIsCreated } ;

  //  Inherited Events
  __property OnClick;
  __property OnDblClick;
  __property OnDragDrop;
  __property OnDragOver;
  __property OnEndDrag;
  __property OnEnter;
  __property OnExit;
  __property OnMouseDown;
  __property OnMouseMove;
  __property OnMouseUp;
  __property OnStartDrag;


   // Overidden Events
   __property TNotifyEvent OnResize = {read=FOnResize,write=FOnResize};
   __property TNotifyEvent OnPaint  = {read=FOnPaint,write=FOnPaint};


   // Added Events
   // Use OnInit to initalize your OpenGL State
   __property TParamsEvent OnSetParams = { read = FOnSetParams , write = FOnSetParams} ;
   __property TNotifyEvent OnSetDisplayMode     = {read=FOnSetDisplayMode,write=FOnSetDisplayMode};
   __property TNotifyEvent OnResetDisplayMode = {read=FOnResetDisplayMode,write=FOnResetDisplayMode};
   __property TNotifyEvent OnRenderInit = {read=FOnRenderInit,write=FOnRenderInit};
   __property TNotifyEvent OnExtensionsInit = {read = FOnExtensionsInit , write = FOnExtensionsInit } ;

public:

   void   __fastcall MakeGLRenderCurrent(void);
   void   __fastcall MakeGLRenderNotCurrent(void);
   void   __fastcall GLRenderSwapBuffers(void);
   bool   __fastcall PanelShareLists(TCustomGLRender *WithOpenGLPanel);
   OpenGLFont3D*  __fastcall Create3DFont(TFont *Font, int FirstGylph, int NumGylph,float Deviation, float Extrusion, int Format);
   void   __fastcall Get3DTextSize(AnsiString Text,GLdouble *XYZ);
   void   __fastcall Get3DTextSize(OpenGLFont3D *Font, AnsiString Text,GLdouble *XYZ);
   void   __fastcall Draw3DText(AnsiString Text);
   void   __fastcall Draw3DText(OpenGLFont3D * Font,AnsiString Text);
   OpenGLFont2D*  __fastcall Create2DFont(TFont *Font, int FirstGylph, int NumGylph);
   void   __fastcall Draw2DText(AnsiString Text);
   void   __fastcall Draw2DText(OpenGLFont2D * Font,AnsiString Text);
   BMPTexture*  __fastcall LoadBMPTexture(AnsiString FileName, Boolean WithAlpha);
   void   __fastcall PrintGLRender(void);
   void   __fastcall SaveOpenGLPanelToFile(const String &FileName);
   bool   __fastcall QueryOpenGLExtention(AnsiString ExtentionName);
   int    __fastcall GetDisplayBPP(void);
   bool   __fastcall SetDisplayBPP(int) ;
   void   __fastcall CreateRenderNow () ;

  __fastcall TCustomGLRender(TComponent* Owner);
  __fastcall ~TCustomGLRender();
};


class PACKAGE TGLRender : public TCustomGLRender
{
public:
  //__fastcall TGLRender(TComponent* Owner) ;
  //__fastcall ~TGLRender();

        // Added Properites
  __property GLExtensions ;
  __property WGLExtensions  ;
  __property UnsupportExtensions   ;
  __property SupportExtensions ;
  __property UserRequestExtensions ;




__published:
   // Inherited Properties
  
   __property Alignment;
   __property Align;
   __property DragCursor;
   __property DragMode;
   __property Enabled;
   __property ParentFont;
   __property ParentShowHint;
   __property PopupMenu;
   __property ShowHint;
   __property TabOrder;
   __property TabStop;
   __property Visible;


   // Added Properites

   __property PixelType ;
   __property PFDFlags ;
   __property cColorBits ;
   __property cDepthBits;
   __property GLSwapBuffers ;
   __property PixelsPerInch ;
   __property TOpenGLPrintScale  ;
   __property Font3D_Enabled ;
   __property Font3D_Type;
   __property Font3D_FirstGylph ;
   __property Font3D_NumGylph ;
   __property Font3D_MaxDeviation ;
   __property Font3D_Extrustion;
   __property Font3D_Format ;
   __property Font2D_Enabled ;
   __property Font2D_Type ;
   __property Font2D_FirstGylph ;
   __property Font2D_NumGylph ;

   __property GLRenderer ;
   __property GLVersion ;
   __property GLVendor ;


   //  Inherited Events
   __property OnClick;
   __property OnDblClick;
   __property OnDragDrop;
   __property OnDragOver;
   __property OnEndDrag;
   __property OnEnter;
   __property OnExit;
   __property OnMouseDown;
   __property OnMouseMove;
   __property OnMouseUp;
   __property OnStartDrag;


   // Overidden Events
   __property OnResize ;
   __property OnPaint ;


   // Added Events
   // Use OnInit to initalize your OpenGL State
    __property OnSetParams  ;
    __property OnSetDisplayMode ;
    __property OnResetDisplayMode ;
    __property OnRenderInit ;
    __property OnExtensionsInit ;

};



//---------------------------------------------------------------------------
#endif
