//---------------------------------------------------------------------------

#include <vcl.h>

#pragma hdrstop


#include "Gadograph.h"

#pragma link "GLRender"
#pragma link "PowerColor"

#pragma package(smart_init)

//---------------------------------------------------------------------------
__fastcall TGadograph::TGadograph(TComponent* Owner)
        : TCustomGLRender(Owner)
{
  //  
  OnRenderInit  = InitGraphRender   ;
  OnPaint       = PaintGraphRender  ;
  OnResize      = ResizeGraphRender ;

  //   
  FStreamList = new TStringList() ;

  //      
  FArgCaptionFormat = new TArgCaptionFormat  ;
  FArgCaptionFormat -> WinControl = this ;
  FArgCaptionFormat -> FloatFormat = ffNumber ;
  FArgCaptionFormat -> Precision = 3 ;
  FArgCaptionFormat -> Digits    = 3 ;

  Font -> Color = clRed ;
  
  //      
  Font -> OnChange = FontChange ;
  ParentFont = false ;
  //    
  FViewPoints = 250 ;
  StreamsCloseLine = false ;
  VHLineColor = clGray ;
  VLine = 5 ;
  HLine = 5 ;
  VHLineWidth = 1 ;
  BackGroundColor=clBlack;
  ArgVisible=true;
  ArgStep = 1 ;
  ArgLocalVisible = true ;

  FPageCount = 0 ;


}
//---------------------------------------------------------------------------
__fastcall TGadograph::~TGadograph()
{
  //  OpenGL 
  if ( FGLCaptionFont )
      delete FGLCaptionFont ;
  
  //    
  for ( int StreamCount = 0  ; StreamCount < FStreamList -> Count ; StreamCount ++  )
    {
      delete (float*)((TGadographStream*)FStreamList -> Objects[StreamCount]) -> XData ;
      delete (float*)((TGadographStream*)FStreamList -> Objects[StreamCount]) -> YData ;
      delete (TGadographStream*)FStreamList -> Objects[StreamCount] ;
    }
 //   
 delete FStreamList ;
}
//---------------------------------------------------------------------------
void __fastcall TGadograph::InitGraphRender ( TObject* Sender )
{

  //  
  FGLCaptionFont = Create2DFont(Font, 0 , 255 ) ;

  float Color[3] ;
  Color2GL ( Color , FBackGroundColor );
  glClearColor ( Color[0] , Color[1] , Color[2] , 1 ) ;
  //   
  glEnable (GL_DEPTH_TEST) ;
  //     RGBA
  glEnable (GL_BLEND) ;
  glBlendFunc ( GL_SRC_ALPHA , GL_ONE_MINUS_SRC_ALPHA) ;

}
//---------------------------------------------------------------------------
void __fastcall TGadograph::ResizeGraphRender ( TObject* Sender )
{
  glViewport(0,0,(GLsizei)Width,(GLsizei)Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //   
  glOrtho(-1.01, 1.01, -1.02, 1.02 , -1.0 , 1.0 ) ;
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
//---------------------------------------------------------------------------
void __fastcall TGadograph::PaintGraphRender ( TObject* Sender )
{
  float StreamsColor[4] = {0.0, 0.0, 0.0, 1.0} ;
  bool LineAttenuation  ;
  float Vertex[3] ;     //      
  Vertex[2] = -1.0 ;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;

  //     
  if ( FVHLineWidth )
   {

     Color2GL ( StreamsColor , FVHLineColor  )  ;
     glColor4fv ( StreamsColor ) ;
     glLineWidth ( FVHLineWidth ) ;
     glBegin(GL_LINES);


     for ( UINT VLineCount = 0 ; VLineCount < FVLine ; VLineCount ++ )
       {
        Vertex[0] = 2*((float)VLineCount/(VLine - 1) - 0.5) ;
        Vertex[1] = -1   ;
        glVertex3fv (Vertex);
        Vertex[1] = 1   ;
        glVertex3fv (Vertex);
       }

     for ( UINT HLineCount = 0 ; HLineCount < FHLine ; HLineCount ++ )
       {
        Vertex[0] = -1   ;
        Vertex[1] = 2*((float)HLineCount/(HLine - 1) - 0.5) ;
        glVertex3fv ( Vertex )  ;
        Vertex[0] = 1   ;
        glVertex3fv ( Vertex )  ;
       }
     glEnd() ;
   }


  for ( int StreamCount = 0 ; StreamCount < FStreamList -> Count ; StreamCount ++ )
    {
      TGadographStream* Stream = (TGadographStream*)FStreamList -> Objects[StreamCount] ;
      //      
      if ( !Stream -> LineWidth ) continue  ;

      Color2GL ( StreamsColor , Stream -> Color) ;
      //   
      if ( Stream -> CaptionVisible )
       {
         glColor3fv ( StreamsColor ) ;
         Vertex[0] = Stream -> XCaptionPos  ;
         Vertex[1] = Stream -> YCaptionPos  ;
         Vertex[2] = 0.5 ;
         glRasterPos3fv (  Vertex ) ;
         Draw2DText ( FGLCaptionFont , Stream -> Caption ) ;
       }
      //   
      if ( Stream -> LineSmooth )
         glEnable (GL_LINE_SMOOTH) ;
      else
         glDisable(GL_LINE_SMOOTH) ;

      glLineWidth ( Stream -> LineWidth ) ;
      glBegin(GL_LINES);
      LineAttenuation = Stream -> Attenuation ;  //   ,     
      StreamsColor[3] = 1.0 ;


      Vertex[2] = -0.75 ;
      for ( UINT Arg = 1 ; Arg < FViewPoints  ; Arg ++ )
       {

        if ( LineAttenuation )
          {
           //     ,    - 
           StreamsColor[3] = (float)((FViewPoints + Arg - FPosition)%FViewPoints)/FViewPoints ;
          }
         glColor4fv ( StreamsColor ) ;
         Vertex[0] = Stream -> XData[ Arg ]   ;
         Vertex[1] = Stream -> YData[ Arg ]  ;
         glVertex3fv ( Vertex ) ;
         Vertex[0] = Stream -> XData[ Arg - 1 ]  ;
         Vertex[1] = Stream -> YData[ Arg - 1 ]  ;
         glVertex3fv ( Vertex ) ;
       }
      glEnd () ;
    }

  //      
  FUtilString = "" ;
  if ( FArgVisible ) FUtilString = FloatToStrF(FArg , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
  if ( FArgLocalVisible )
     if (FUtilString.IsEmpty()) FUtilString = FloatToStrF(FPosition * FArgStep  , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
     else FUtilString += "/" + FloatToStrF(FPosition * FArgStep  , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
  if ( FUtilString.IsEmpty() ) return ;
  StreamsColor[3] = 1;
  Color2GL ( StreamsColor ,  Font -> Color ) ;
  glColor3fv ( StreamsColor ) ;
  Vertex[0] = FArgLabelXPos  ;
  Vertex[1] = FArgLabelYPos  ;
  Vertex[2] = 0.0  ;
  glRasterPos3fv ( Vertex ) ;
  Draw2DText ( FGLCaptionFont , FUtilString + FArgUnitName ) ;
}
//--------------------------------------------------------------------------
void __fastcall TGadograph::SetBackGroundColor (TColor val)
{
  //   
  FBackGroundColor = val ;
  if ( !IsCreated ) return ;
  // 
  /*MakeGLRenderCurrent () ;
  float Color[3] ;
  Color2GL ( Color , FBackGroundColor );
  glClearColor ( Color[0] , Color[1] , Color[2] ,1 ) ;
  MakeGLRenderNotCurrent() ;
  Repaint() ; */
}
//-------------------------------------------------------------------------
unsigned int __fastcall TGadograph::GetStreamNum()
{
  return FStreamList -> Count ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetViewPoints(unsigned int value)
{
  //        
  for ( int SeriesCount = 0  ; SeriesCount < FStreamList -> Count ; SeriesCount ++  )
    {
      float* XData = new float [value] ;
      float* YData = new float [value] ;

      if ( !XData || !YData ) throw Exception( "Failure for memory reallocated data stream array" )  ;

      delete (float*)((TGadographStream*)FStreamList -> Objects[SeriesCount]) -> XData ;
      delete (float*)((TGadographStream*)FStreamList -> Objects[SeriesCount]) -> YData ;

      memset ( XData , 0 , value * sizeof(float) ) ;
      memset ( YData , 0 , value * sizeof(float) ) ;
      ((TGadographStream*)FStreamList -> Objects[SeriesCount]) -> XData = XData ;
      ((TGadographStream*)FStreamList -> Objects[SeriesCount]) -> YData = YData ;
    }
  FViewPoints = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
unsigned int __fastcall TGadograph::GetViewPoints()
{
  return FViewPoints ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::AddStream(AnsiString StreamName  ,  //  
                                      TColor     Color       ,  //   
                                      bool       Attenuation ,  //   
                                      UINT       LineWidth   ,  //   
                                      bool       LineSmooth     //   
                                     )
{
  if ( FStreamList -> IndexOf ( StreamName ) != -1 )
        throw Exception( "Stream with name \"" + StreamName + "\"  is alredy exist" ) ;

  //     
  float* XData = new float[FViewPoints] ;
  float* YData = new float[FViewPoints] ;
  if ( !XData  || !YData ) throw Exception( "Failure for memory allocated data stream array" )  ;
  memset ( XData , 0 , FViewPoints * sizeof(float)  ) ;
  memset ( YData , 0 , FViewPoints * sizeof(float)  ) ;

  TGadographStream* Stream = new TGadographStream ;
  if ( !Stream ) throw Exception( "Failure for memory reallocated stream structure" )  ;

  Stream -> Color       = Color       ;
  Stream -> Attenuation = Attenuation ;
  Stream -> LineWidth   = LineWidth   ;
  Stream -> LineSmooth  = LineSmooth  ;
  Stream -> XData = XData ;
  Stream -> YData = YData ;

  FStreamList -> AddObject( StreamName , (TObject*)Stream ) ;
  Repaint() ;

}
//-------------------------------------------------------------------------
void __fastcall TGadograph::DeleteStream(AnsiString StreamName )
{
  int StreamIndex = FStreamList -> IndexOf ( StreamName ) ;
  if ( FStreamList -> IndexOf ( StreamName ) == -1 )
        throw Exception( "Stream with name \"" + StreamName + "\" is not exist" ) ;

  TGadographStream* Stream = (TGadographStream*)FStreamList -> Objects[ StreamIndex ] ;
  delete Stream -> XData ;
  delete Stream -> YData ;
  delete Stream ;

  FStreamList -> Delete( StreamIndex ) ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::UpdateStreams( float* vals )
{
  //     
  FArgCount ++ ;
  FArg += FArgStep ;
  FPosition = (++FPosition ) % FViewPoints ;

  for ( int StreamCount = 0 ; StreamCount < FStreamList -> Count ; StreamCount ++)
    {
      ((TGadographStream* )FStreamList -> Objects[StreamCount]) -> XData [ FPosition ]  =  vals[StreamCount*2 + 0] ;
      ((TGadographStream* )FStreamList -> Objects[StreamCount]) -> YData [ FPosition ]  =  vals[StreamCount*2 + 1] ;
    }
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetColor( UINT index, TColor value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> Color = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
TColor  __fastcall TGadograph::GetColor( UINT index)
{
 return ((TGadographStream*)FStreamList -> Objects[index] ) -> Color ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetAttenuation( UINT index, bool value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> Attenuation = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TGadograph::GetAttenuation( UINT index)
{
 return ((TGadographStream*)FStreamList -> Objects[index] ) -> Attenuation ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetLineWidth (UINT index, UINT value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> LineWidth = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
UINT  __fastcall TGadograph::GetLineWidth (UINT index)
{
 return ((TGadographStream*)FStreamList -> Objects[index] ) -> LineWidth  ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetLineSmooth (UINT index, bool value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> LineSmooth = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TGadograph::GetLineSmooth (UINT index)
{
 return ((TGadographStream*)FStreamList -> Objects[index] ) -> LineSmooth ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetCaptionVisible (UINT index, bool value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> CaptionVisible = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TGadograph::GetCaptionVisible (UINT index)
{
 return ((TGadographStream*)FStreamList -> Objects[index] ) -> CaptionVisible ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetXCaptionPos (UINT index,  float value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> XCaptionPos = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TGadograph::GetXCaptionPos (UINT index )
{
 return  ((TGadographStream*)FStreamList -> Objects[index] ) -> XCaptionPos  ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetYCaptionPos (UINT index,  float value )
{
 ((TGadographStream*)FStreamList -> Objects[index] ) -> YCaptionPos = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TGadograph::GetYCaptionPos (UINT index )
{
 return  ((TGadographStream*)FStreamList -> Objects[index] ) -> YCaptionPos  ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetVLine (UINT value)
{
 if ( value == 1 )
    FVLine = 2 ;
 else
    FVLine = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetHLine (UINT value)
{
 if ( value == 1 )
    FHLine = 2 ;
 else
    FHLine = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetVHLineWidth (UINT value)
{
  FVHLineWidth = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetVHLineColor (TColor value)
{
  FVHLineColor = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetStreamsCloseLine (bool value)
{
  FStreamsCloseLine = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgVisible(bool value)
{
 FArgVisible = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgUnitName(AnsiString value)
{
 FArgUnitName = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgStep(float value)
{
 FArgStep = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArg(float value)
{
 FArg = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgCount(unsigned __int64 value)
{
 FArgCount = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
TGadographStream* __fastcall TGadograph::GetStream (UINT index)
{
 return (TGadographStream*)FStreamList ->Objects[index] ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::SetCaption (UINT index,  AnsiString value )
{
  ((TGadographStream*)FStreamList ->Objects[index]) -> Caption = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
AnsiString __fastcall TGadograph::GetCaption (UINT index )
{
 return ((TGadographStream*)FStreamList ->Objects[index]) -> Caption ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgLabelXPos(float value)
{
 FArgLabelXPos = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgLabelYPos(float value)
{
 FArgLabelYPos = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgLocalVisible(bool value)
{
 FArgLocalVisible = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TGadograph::FontChange ( TObject* Sender )
{
  if ( !IsCreated ) return ;
  delete  FGLCaptionFont ;
 /* MakeGLRenderCurrent() ;
  FGLCaptionFont = Create2DFont(Font, 0 , 255 ) ;
  MakeGLRenderNotCurrent() ;
  Repaint() ;   */
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetArgCaptionFormat ( TArgCaptionFormat* value )
{
 delete FArgCaptionFormat ;
 FArgCaptionFormat = value ;
}
//-------------------------------------------------------------------------
float __fastcall TGadograph::GetArgLocal ()
{
 return ((float)FPosition) * FArgStep  ;
}
//-------------------------------------------------------------------------
void __fastcall TGadograph::SetPageCount(UINT value)
{
  FPageCount = value ;
}
