//---------------------------------------------------------------------------

#ifndef GadographH
#define GadographH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>

#include <Controls.hpp>
#include <ExtCtrls.hpp>

#include "PowerColor.h"
#include "GLRender.h"
#include "StreamGraph.h"

typedef struct _tagTGadographStream
   {
     float*     XData       ;  //   X
     float*     YData       ;  //   Y
     TColor     Color       ;  //   
     bool       Attenuation ;  //   

     UINT       LineWidth   ;  //   
     bool       LineSmooth  ;  //   

     bool       CaptionVisible;//      
     float      XCaptionPos ;  // X   
     float      YCaptionPos ;  // Y   
     AnsiString Caption     ;  // 

   } TGadographStream ;
//---------------------------------------------------------------------------
class PACKAGE TGadograph : public TCustomGLRender
{
private:
  UINT         FPosition   ;       //      
  TColor       FBackGroundColor ;  //  
  TStringList* FStreamList ;       //     
  UINT         FViewPoints ;       //       
  UINT         FVLine          ;   //     
  UINT         FHLine          ;   //     
  UINT         FVHLineWidth    ;   //    
  TColor       FVHLineColor   ;    //   
  bool         FStreamsCloseLine ; //      
  UINT         FPageCount   ; //    
  bool         FArgVisible   ;     //    
  bool         FArgLocalVisible ;  //       ( LocalArg = Arg % ViewPoints )
  float        FArgLabelXPos ;     // X    
  float        FArgLabelYPos ;     // Y    
  AnsiString   FArgUnitName      ; //     
  float        FArgStep          ; //     
  float        FArg              ; //    
  unsigned __int64  FArgCount    ; //      
  TArgCaptionFormat*  FArgCaptionFormat ; //   
  OpenGLFont2D* FGLCaptionFont   ; // OpenGL  


  AnsiString   FUtilString       ;


  unsigned int __fastcall GetStreamNum();

  void __fastcall SetViewPoints(UINT value);
  UINT __fastcall GetViewPoints();


  void  __fastcall SetMaxCriticalValue( UINT index, float value );
  float __fastcall GetMaxCriticalValue( UINT index);

  void  __fastcall SetMinCriticalValue( UINT index, float value );
  float __fastcall GetMinCriticalValue( UINT index);

  void  __fastcall SetColor( UINT index, TColor value );
  TColor  __fastcall GetColor( UINT index);

  void  __fastcall SetAutoNormalize( UINT index, bool value );
  bool  __fastcall GetAutoNormalize( UINT index);

  void  __fastcall SetAttenuation (UINT index, bool value );
  bool  __fastcall GetAttenuation (UINT index);

  void  __fastcall SetLineSmooth (UINT index, bool value );
  bool  __fastcall GetLineSmooth (UINT index);

  void  __fastcall SetLineWidth (UINT index, UINT value );
  UINT  __fastcall GetLineWidth (UINT index);

  void  __fastcall SetCaptionVisible (UINT index, bool value );
  bool  __fastcall GetCaptionVisible (UINT index);

  void  __fastcall SetData (UINT index, UINT arg , float value );
  float __fastcall GetData (UINT index, UINT arg );

  void  __fastcall SetXCaptionPos (UINT index,  float value );
  float __fastcall GetXCaptionPos (UINT index );

  void  __fastcall SetYCaptionPos (UINT index,  float value );
  float __fastcall GetYCaptionPos (UINT index );

  void  __fastcall SetCaption (UINT index,  AnsiString value );
  AnsiString __fastcall GetCaption (UINT index );

  TGadographStream* __fastcall GetStream (UINT index) ;
  float __fastcall GetArgLocal () ;

  
  void  __fastcall SetBackGroundColor (TColor value) ;
  void  __fastcall SetFrontLineColor(TColor value);
  void  __fastcall SetFrontLineWidth (UINT value) ;
  void  __fastcall SetVLine (UINT value) ;
  void  __fastcall SetHLine (UINT value) ;
  void  __fastcall SetVHLineWidth (UINT value) ;
  void  __fastcall SetVHLineColor (TColor value) ;
  void  __fastcall SetStreamsCloseLine (bool value) ;
  void  __fastcall SetArgVisible(bool value);
  void  __fastcall SetArgUnitName(AnsiString value);
  void  __fastcall SetArgStep(float value);
  void  __fastcall SetArg(float value);
  void  __fastcall SetArgCount(unsigned __int64 value);
  void  __fastcall SetArgLabelXPos(float value);
  void  __fastcall SetArgLabelYPos(float value);
  void  __fastcall SetArgLocalVisible(bool value);
  void  __fastcall SetArgCaptionFormat ( TArgCaptionFormat* value );


  void  __fastcall FontChange ( TObject* Sender ) ;
  void __fastcall SetPageCount(UINT value);

protected:

  void __fastcall InitGraphRender   ( TObject* Sender ) ;
  void __fastcall PaintGraphRender  ( TObject* Sender ) ;
  void __fastcall ResizeGraphRender ( TObject* Sender ) ;

public:
  __fastcall  TGadograph(TComponent* Owner);
  __fastcall ~TGadograph();

  void __fastcall AddStream(AnsiString StreamName  ,  //  
                            TColor     Color       ,  //   
                            bool       Attenuation ,  //   
                            UINT       LineWidth   ,  //   
                            bool       LineSmooth     //   
                            );


   void __fastcall DeleteStream(AnsiString StreamName );
   void __fastcall UpdateStreams( float* vals );

  //  

  __property unsigned int  StreamNum  = { read=GetStreamNum};
  __property TGadographStream* Streams[UINT index] = {read= GetStream } ;

  __property float  Data        [UINT index][UINT arg] = { read = GetData , write = SetData     }  ;
  __property UINT   LineWidth   [UINT index] = { read = GetLineWidth   , write = SetLineWidth   }  ;
  __property float  MaxCriticalValue    [UINT index] = { read = GetMaxCriticalValue    , write = SetMaxCriticalValue   }  ;
  __property float  MinCriticalValue    [UINT index] = { read = GetMinCriticalValue    , write = SetMinCriticalValue   }  ;
  __property TColor Color       [UINT index] = { read = GetColor       , write = SetColor       }  ;
  __property bool   AutoNormalize   [UINT index] = { read = GetAutoNormalize   , write = SetAutoNormalize   }  ;
  __property bool   Attenuation [UINT index] = { read = GetAttenuation , write = SetAttenuation }  ;
  __property bool   LineSmooth  [UINT index] = { read = GetLineSmooth  , write = SetLineSmooth  }  ;
  __property bool   CaptionVisible[UINT index] = { read = GetCaptionVisible, write = SetCaptionVisible}  ;
  __property float  XCaptionPos [UINT index] = { read = GetXCaptionPos , write = SetXCaptionPos }  ;
  __property float  YCaptionPos [UINT index] = { read = GetYCaptionPos , write = SetYCaptionPos }  ;
  __property AnsiString Caption [UINT index] = { read = GetCaption     , write = SetCaption     }  ;
  __property float  ArgLocal  = { read=GetArgLocal };

  __property UINT PageCount  = { read=FPageCount, write=SetPageCount };


__published:
  //  

   __property Font ;
   __property Alignment;
   __property Align;
   __property DragCursor;
   __property DragMode;
   __property Enabled;
   __property ParentFont;
   __property ParentShowHint;
   __property PopupMenu;
   __property ShowHint;
   __property TabOrder;
   __property TabStop;
   __property Visible;

   __property OnClick;
   __property OnDblClick;
   __property OnDragDrop;
   __property OnDragOver;
   __property OnEndDrag;
   __property OnEnter;
   __property OnExit;
   __property OnMouseDown;
   __property OnMouseMove;
   __property OnMouseUp;
   __property OnStartDrag;


    //  
  __property bool StreamsCloseLine = {read = FStreamsCloseLine , write = SetStreamsCloseLine , default=false }  ;
  __property TColor VHLineColor = {read = FVHLineColor , write = SetVHLineColor , default=clGray }  ;
  __property UINT VLine = {read = FVLine , write = SetVLine , default = 5 }  ;
  __property UINT HLine = {read = FHLine , write = SetHLine , default = 5 }  ;
  __property UINT VHLineWidth = {read = FVHLineWidth , write = SetVHLineWidth , default=1 }  ;
  __property TColor BackGroundColor = {read = FBackGroundColor , write = SetBackGroundColor , default=clBlack}  ;
  __property UINT ViewPoints  = { read=GetViewPoints, write=SetViewPoints , default=250};
  __property bool ArgVisible  = { read=FArgVisible, write=SetArgVisible , default=true};
  __property AnsiString ArgUnitName  = { read=FArgUnitName, write=SetArgUnitName };
  __property float ArgStep  = { read = FArgStep, write=SetArgStep , default=1};
  __property float Arg  = { read=FArg, write=SetArg };
  __property float ArgLabelXPos  = { read=FArgLabelXPos, write=SetArgLabelXPos };
  __property float ArgLabelYPos  = { read=FArgLabelYPos, write=SetArgLabelYPos };
  __property bool  ArgLocalVisible  = { read=FArgLocalVisible, write=SetArgLocalVisible, default=true };
  __property TArgCaptionFormat* ArgCaptionFormat  = { read=FArgCaptionFormat , write=SetArgCaptionFormat };
  __property unsigned __int64 ArgCount  = { read=FArgCount, write=SetArgCount };

};
//---------------------------------------------------------------------------

#endif
