//---------------------------------------------------------------------------

#include <vcl.h>

#pragma hdrstop


#include "StreamGraph.h"

#pragma link "GLRender"
#pragma link "PowerColor"

#pragma package(smart_init)

//---------------------------------------------------------------------------
__fastcall TStreamGraph::TStreamGraph(TComponent* Owner)
        : TCustomGLRender(Owner)
{
  //  
  OnRenderInit  = InitStreamGraphRender   ;
  OnPaint       = PaintStreamGraphRender  ;
  OnResize      = ResizeStreamGraphRender ;

  //   
  FStreamList = new TStringList() ;

  //      
  FArgCaptionFormat = new TArgCaptionFormat  ;
  FArgCaptionFormat -> WinControl = this ;
  FArgCaptionFormat -> FloatFormat = ffNumber ;
  FArgCaptionFormat -> Precision = 3 ;
  FArgCaptionFormat -> Digits    = 3 ;

  Font -> Color = clRed ;
  
  //      
  Font -> OnChange = FontChange ;
  ParentFont = false ;
  //    
  FViewPoints = 250 ;
  StreamsCloseLine = false ;
  VHLineColor = clGray ;
  VLine = 5 ;
  HLine = 5 ;
  VHLineWidth = 1 ;
  FrontLineWidth=1;
  FrontLineColor= clGray ;
  BackGroundColor=clBlack;
  ArgVisible=true;
  ArgStep = 1 ;
  ArgLocalVisible = true ;

  FPageCount = 0 ;


}
//---------------------------------------------------------------------------
__fastcall TStreamGraph::~TStreamGraph()
{
  //  OpenGL 
  if ( FGLCaptionFont )
      delete FGLCaptionFont ;
  
  //    
  for ( int StreamCount = 0  ; StreamCount < FStreamList -> Count ; StreamCount ++  )
    {
      delete (float*)((TGraphStream*)FStreamList -> Objects[StreamCount]) -> Data ;
      delete (TGraphStream*)FStreamList -> Objects[StreamCount] ;
    }
 //   
 delete FStreamList ;
}
//---------------------------------------------------------------------------
void __fastcall TStreamGraph::InitStreamGraphRender ( TObject* Sender )
{

  //  
  FGLCaptionFont = Create2DFont(Font, 0 , 255 ) ;

  float Color[3] ;
  Color2GL ( Color , FBackGroundColor );
  glClearColor ( Color[0] , Color[1] , Color[2] , 1 ) ;
  //   
  glEnable (GL_DEPTH_TEST) ;
  //     RGBA
  glEnable (GL_BLEND) ;
  glBlendFunc ( GL_SRC_ALPHA , GL_ONE_MINUS_SRC_ALPHA) ;

}
//---------------------------------------------------------------------------
void __fastcall TStreamGraph::ResizeStreamGraphRender ( TObject* Sender )
{
  glViewport(0,0,(GLsizei)Width,(GLsizei)Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //   
  glOrtho(-1.01, 1.01, -1.02, 1.02 , -1.0 , 1.0 ) ;
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
//---------------------------------------------------------------------------
void __fastcall TStreamGraph::PaintStreamGraphRender ( TObject* Sender )
{
  float StreamsColor[4] = {0.0, 0.0, 0.0, 1.0} ;
  bool LineAttenuation  ;
  float Vertex[3] ;     //      
  Vertex[2] = -1.0 ;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;

  //     
  if ( FVHLineWidth )
   {

     Color2GL ( StreamsColor , FVHLineColor  )  ;
     glColor4fv ( StreamsColor ) ;
     glLineWidth ( FVHLineWidth ) ;
     glBegin(GL_LINES);


     for ( UINT VLineCount = 0 ; VLineCount < FVLine ; VLineCount ++ )
       {
        Vertex[0] = 2*((float)VLineCount/(VLine - 1) - 0.5) ;
        Vertex[1] = -1   ;
        glVertex3fv (Vertex);
        Vertex[1] = 1   ;
        glVertex3fv (Vertex);
       }

     for ( UINT HLineCount = 0 ; HLineCount < FHLine ; HLineCount ++ )
       {
        Vertex[0] = -1   ;
        Vertex[1] = 2*((float)HLineCount/(HLine - 1) - 0.5) ;
        glVertex3fv ( Vertex )  ;
        Vertex[0] = 1   ;
        glVertex3fv ( Vertex )  ;
       }
     glEnd() ;
   }


  for ( int StreamCount = 0 ; StreamCount < FStreamList -> Count ; StreamCount ++ )
    {
      TGraphStream* Stream = (TGraphStream*)FStreamList -> Objects[StreamCount] ;
      //      
      if ( !Stream -> LineWidth ) continue  ;

      Color2GL ( StreamsColor , Stream -> Color) ;
      //   
      if ( Stream -> CaptionVisible )
       {
         glColor3fv ( StreamsColor ) ;
         Vertex[0] = Stream -> XCaptionPos  ;
         Vertex[1] = Stream -> YCaptionPos  ;
         Vertex[2] = 0.5 ;
         glRasterPos3fv (  Vertex ) ;
         Draw2DText ( FGLCaptionFont , Stream -> Caption ) ;
       }
      //   
      if ( Stream -> LineSmooth )
         glEnable (GL_LINE_SMOOTH) ;
      else
         glDisable(GL_LINE_SMOOTH) ;

      glLineWidth ( Stream -> LineWidth ) ;
      glBegin(GL_LINES);
      LineAttenuation = Stream -> Attenuation ;  //   ,     
      StreamsColor[3] = 1.0 ;
      

      Vertex[2] = -0.75 ;
      for ( UINT Arg = 0 ; Arg < FViewPoints - 1; Arg ++ )
       {
        //      
        if ( !FStreamsCloseLine )
          if ( Arg == FPosition ) continue ;
        if ( LineAttenuation )
          {
           //     ,    - 
           StreamsColor[3] = (float)((FViewPoints + Arg - FPosition)%FViewPoints)/FViewPoints ;
          }
         glColor4fv ( StreamsColor ) ;

         Vertex[0] = 2*(((float)Arg ) / FViewPoints    - 0.5)  ;
         Vertex[1] = Stream -> Data[ Arg ]  ;
         glVertex3fv ( Vertex ) ;
         Vertex[0] = 2*((float)(Arg + 1) / FViewPoints - 0.5)  ;
         Vertex[1] = Stream -> Data[ Arg + 1]  ;
         glVertex3fv ( Vertex ) ;
       }
      glEnd () ;
    }


  //     
  Vertex[2] = -0.25 ;
  if ( FFrontLineWidth  )
   {
     glEnable(GL_LINE_SMOOTH) ;
     StreamsColor[3] = 1.0 ;
     glLineWidth ( FFrontLineWidth ) ;
     Color2GL ( StreamsColor , FFrontLineColor ) ;
     glColor4fv (StreamsColor) ;
     glBegin(GL_LINES);
       Vertex[0] = 2*((float)(FPosition + 0.5 ) / FViewPoints - 0.5 )  ;
       Vertex[1] = 1  ;
       glVertex3fv( Vertex ) ;
       Vertex[1] = -1  ;
       glVertex3fv( Vertex ) ;
     glEnd () ;
     glDisable(GL_LINE_SMOOTH) ;
   }


  //      
  FUtilString = "" ;
  if ( FArgVisible ) FUtilString = FloatToStrF(FArg , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
  if ( FArgLocalVisible )
     if (FUtilString.IsEmpty()) FUtilString = FloatToStrF(FPosition * FArgStep  , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
     else FUtilString += "/" + FloatToStrF(FPosition * FArgStep  , FArgCaptionFormat -> FloatFormat , FArgCaptionFormat -> Precision , FArgCaptionFormat -> Digits) ;
  if ( FUtilString.IsEmpty() ) return ;
  StreamsColor[3] = 1;
  Color2GL ( StreamsColor ,  Font -> Color ) ;
  glColor3fv ( StreamsColor ) ;
  Vertex[0] = FArgLabelXPos  ;
  Vertex[1] = FArgLabelYPos  ;
  Vertex[2] = 0.0  ;
  glRasterPos3fv ( Vertex ) ;
  Draw2DText ( FGLCaptionFont , FUtilString + FArgUnitName ) ;
}
//--------------------------------------------------------------------------
void __fastcall TStreamGraph::SetBackGroundColor (TColor val)
{
  //   
  FBackGroundColor = val ;
  if ( !IsCreated ) return ;
  // 
 /* MakeGLRenderCurrent () ;
  float Color[3] ;
  Color2GL ( Color , FBackGroundColor );
  glClearColor ( Color[0] , Color[1] , Color[2] ,1 ) ;
  MakeGLRenderNotCurrent() ;
  Repaint() ;       */
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetFrontLineColor (TColor val)
{
  FFrontLineColor = val ;
  Repaint() ;
}
//-------------------------------------------------------------------------
unsigned int __fastcall TStreamGraph::GetStreamNum()
{
  return FStreamList -> Count ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetViewPoints(unsigned int value)
{
  //        
  for ( int SeriesCount = 0  ; SeriesCount < FStreamList -> Count ; SeriesCount ++  )
    {
      float* Data = new float [value] ;
      if ( !Data ) throw Exception( "Failure for memory reallocated data stream array" )  ;

      //memcpy ( Data , ((TGraphStream*)FStreamList -> Objects[SeriesCount]) -> Data , ViewPoints * sizeof(float) ) ;
      delete (float*)((TGraphStream*)FStreamList -> Objects[SeriesCount]) -> Data ;
      memset ( Data , 0 , value * sizeof(float) ) ;
      ((TGraphStream*)FStreamList -> Objects[SeriesCount]) -> Data = Data ;
    }
  FViewPoints = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
unsigned int __fastcall TStreamGraph::GetViewPoints()
{
  return FViewPoints ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::AddStream(AnsiString StreamName  ,  //  
                                        float      MaxCriticalValue    ,  //     
                                        float      MinCriticalValue    ,  //      
                                        bool       AutoNormalize   ,  //       
                                        TColor     Color       ,  //   
                                        bool       Attenuation ,  //   
                                        UINT       LineWidth   ,  //   
                                        bool       LineSmooth  ,  //   
                                        bool       Notifiable     //         
                            )
{
  if ( FStreamList -> IndexOf ( StreamName ) != -1 )
        throw Exception( "Stream with name \"" + StreamName + "\"  is alredy exist" ) ;

  //     
  float* Data = new float[FViewPoints] ;
  if ( !Data ) throw Exception( "Failure for memory allocated data stream array" )  ;
  memset ( Data , 0 , FViewPoints * sizeof(float)  ) ;

  TGraphStream* Stream = new TGraphStream ;
  if ( !Stream ) throw Exception( "Failure for memory reallocated stream structure" )  ;

  Stream -> MaxCriticalValue   = MaxCriticalValue    ;
  Stream -> MinCriticalValue   = MinCriticalValue    ;
  Stream -> AutoNormalize   = AutoNormalize   ;
  Stream -> Color       = Color       ;
  Stream -> Attenuation = Attenuation ;
  Stream -> LineWidth   = LineWidth   ;
  Stream -> LineSmooth  = LineSmooth  ;
  Stream -> Notifiable  = Notifiable  ;
  Stream -> Data = Data ;

  FStreamList -> AddObject( StreamName , (TObject*)Stream ) ;
  Repaint() ;

}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::DeleteStream(AnsiString StreamName )
{
  int StreamIndex = FStreamList -> IndexOf ( StreamName ) ;
  if ( FStreamList -> IndexOf ( StreamName ) == -1 )
        throw Exception( "Stream with name \"" + StreamName + "\" is not exist" ) ;

  TGraphStream* Stream = (TGraphStream*)FStreamList -> Objects[ StreamIndex ] ;
  delete Stream -> Data ;
  delete Stream ;

  FStreamList -> Delete( StreamIndex ) ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::UpdateStreams( float* vals )
{
  //     
  FArgCount ++ ;
  FArg += FArgStep ;
  FPosition = (FPosition + 1) % FViewPoints ;

  //    
  if ( !FPosition )
   {
    FPageCount ++ ;
    if ( FOnSwapPage ) FOnSwapPage ( this , FPageCount) ;
   }

  //     
  float LastVal , PrevVal;
  float MaxCriticalValue , MinCriticalValue , *PrevMaxCriticalValue , *PrevMinCriticalValue ;
  bool  IsOverLevel  ;
  for ( int StreamCount = 0 ; StreamCount < FStreamList -> Count ; StreamCount ++)
    {
     TGraphStream* Stream = (TGraphStream* )FStreamList -> Objects[StreamCount] ;
     LastVal  = vals[StreamCount] ;
     PrevVal = Stream -> Data[(FPosition-1) % FViewPoints] ;
     MaxCriticalValue = Stream -> MaxCriticalValue ;
     MinCriticalValue = Stream -> MinCriticalValue ;
     PrevMaxCriticalValue = &Stream -> PrevMaxCriticalValue ;
     PrevMinCriticalValue = &Stream -> PrevMinCriticalValue ;

     //     
     if  ( LastVal > MaxCriticalValue ) Stream -> StreamLevelState = slsMaxCriticalOut ;
     if  ( LastVal < MinCriticalValue ) Stream -> StreamLevelState = slsMinCriticalOut ;

     //        
     if ( FOnOutMaxCritical && ((TGraphStream* )FStreamList -> Objects[StreamCount]) -> Notifiable )
        {
          if ( (LastVal > MaxCriticalValue) && (PrevVal <= *PrevMaxCriticalValue) )  FOnOutMaxCritical (this , StreamCount ) ;
          *PrevMaxCriticalValue = MaxCriticalValue ;
        }

     //         
     if ( FOnOverMaxCritical && ((TGraphStream* )FStreamList -> Objects[StreamCount]) -> Notifiable )
        {
          if ( (LastVal <= MaxCriticalValue) && (PrevVal > *PrevMaxCriticalValue) )  FOnOverMaxCritical (this , StreamCount ) ;
          *PrevMaxCriticalValue = MaxCriticalValue ;
        }

     //         
     if ( FOnOutMinCritical && ((TGraphStream* )FStreamList -> Objects[StreamCount]) -> Notifiable )
        {
          if ( LastVal < MinCriticalValue && PrevVal >= *PrevMinCriticalValue )  FOnOutMinCritical (this , StreamCount ) ;
          *PrevMinCriticalValue = MinCriticalValue ;
        }

     //        
     if ( FOnOverMinCritical && ((TGraphStream* )FStreamList -> Objects[StreamCount]) -> Notifiable )
        {
          if ( LastVal >= MinCriticalValue && PrevVal < *PrevMinCriticalValue )  FOnOverMinCritical (this , StreamCount ) ;
          *PrevMinCriticalValue = MinCriticalValue ;
        }

     ((TGraphStream* )FStreamList -> Objects[StreamCount]) -> Data [ FPosition ]  =  LastVal ;
    }
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetMaxCriticalValue( UINT index, float value)
{
 if ( ((TGraphStream*)FStreamList -> Objects[index] ) -> MinCriticalValue > value )
   throw Exception ("MaxCriticalValue mast be greater then MinCriticalValue\r\nor equal" ) ;
 ((TGraphStream*)FStreamList -> Objects[index] ) -> MaxCriticalValue = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetMaxCriticalValue( UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> MaxCriticalValue ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetMinCriticalValue( UINT index, float value)
{
 if ( ((TGraphStream*)FStreamList -> Objects[index] ) -> MaxCriticalValue < value )
   throw Exception ("MinCriticalValue mast be less then MaxCriticalValue\r\nor equal" ) ;
 ((TGraphStream*)FStreamList -> Objects[index] ) -> MinCriticalValue = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetMinCriticalValue( UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> MinCriticalValue ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetAutoNormalize( UINT index, bool value)
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> AutoNormalize = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool __fastcall TStreamGraph::GetAutoNormalize( UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> AutoNormalize ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetColor( UINT index, TColor value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> Color = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
TColor  __fastcall TStreamGraph::GetColor( UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> Color ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetAttenuation( UINT index, bool value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> Attenuation = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TStreamGraph::GetAttenuation( UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> Attenuation ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetLineWidth (UINT index, UINT value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> LineWidth = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
UINT  __fastcall TStreamGraph::GetLineWidth (UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> LineWidth  ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetLineSmooth (UINT index, bool value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> LineSmooth = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TStreamGraph::GetLineSmooth (UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> LineSmooth ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetCaptionVisible (UINT index, bool value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> CaptionVisible = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
bool  __fastcall TStreamGraph::GetCaptionVisible (UINT index)
{
 return ((TGraphStream*)FStreamList -> Objects[index] ) -> CaptionVisible ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetXCaptionPos (UINT index,  float value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> XCaptionPos = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetXCaptionPos (UINT index )
{
 return  ((TGraphStream*)FStreamList -> Objects[index] ) -> XCaptionPos  ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetYCaptionPos (UINT index,  float value )
{
 ((TGraphStream*)FStreamList -> Objects[index] ) -> YCaptionPos = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetYCaptionPos (UINT index )
{
 return  ((TGraphStream*)FStreamList -> Objects[index] ) -> YCaptionPos  ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetFrontLineWidth (UINT value)
{
 FFrontLineWidth  = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetVLine (UINT value)
{
 if ( value == 1 )
    FVLine = 2 ;
 else
    FVLine = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetHLine (UINT value)
{
 if ( value == 1 )
    FHLine = 2 ;
 else
    FHLine = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetVHLineWidth (UINT value)
{
  FVHLineWidth = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetVHLineColor (TColor value)
{
  FVHLineColor = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetStreamsCloseLine (bool value)
{
  FStreamsCloseLine = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetData (UINT index, UINT arg , float value )
{
  ((TGraphStream*)FStreamList -> Objects[index] ) -> Data[arg] = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetData (UINT index , UINT arg )
{
  return ((TGraphStream*)FStreamList -> Objects[index] ) -> Data[arg]  ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgVisible(bool value)
{
 FArgVisible = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgUnitName(AnsiString value)
{
 FArgUnitName = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgStep(float value)
{
 FArgStep = value ;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArg(float value)
{
 FArg = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgCount(unsigned __int64 value)
{
 FArgCount = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
TGraphStream* __fastcall TStreamGraph::GetStream (UINT index)
{
 return (TGraphStream*)FStreamList ->Objects[index] ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::SetCaption (UINT index,  AnsiString value )
{
  ((TGraphStream*)FStreamList ->Objects[index]) -> Caption = value ;
  Repaint() ;
}
//-------------------------------------------------------------------------
AnsiString __fastcall TStreamGraph::GetCaption (UINT index )
{
 return ((TGraphStream*)FStreamList ->Objects[index]) -> Caption ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgLabelXPos(float value)
{
 FArgLabelXPos = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgLabelYPos(float value)
{
 FArgLabelYPos = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgLocalVisible(bool value)
{
 FArgLocalVisible = value;
 Repaint() ;
}
//-------------------------------------------------------------------------
void  __fastcall TStreamGraph::FontChange ( TObject* Sender )
{
  /*if ( !IsCreated ) return ;
  delete  FGLCaptionFont ;
  MakeGLRenderCurrent() ;
  FGLCaptionFont = Create2DFont(Font, 0 , 255 ) ;
  MakeGLRenderNotCurrent() ;
  Repaint() ;  */
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetArgCaptionFormat ( TArgCaptionFormat* value )
{
 delete FArgCaptionFormat ;
 FArgCaptionFormat = value ;
}
//-------------------------------------------------------------------------
float __fastcall TStreamGraph::GetArgLocal ()
{
 return ((float)FPosition) * FArgStep  ;
}
//-------------------------------------------------------------------------
void __fastcall TStreamGraph::SetPageCount(UINT value)
{
  FPageCount = value ;
}
